<?php

// ViaThinkSoft Downloading functions for PHP
// Licensed under GPLv3

// Note: system() is still in torified environment if you run this library in "vtor".
// However, "vtor" does not work if you use file_get_contents() or the inbuilt cURL etc.

// TODO: header dumpen und im fehlerfall in der lage sein den HTTP header auszugeben -- problem, vdown macht das nicht
// TODO: keine globalen funktionen getLastError() etc (berall), sondern return arguments...

class VTSDownloader {

	# --- Construction

	protected $vtor_cmd  = '';

	public function __construct() {
		$this->vtor_cmd  = __DIR__ . '/vtor';
	}

	# --- Configuration: force_tor - Default 0 (Note: requires 'vtor')

	private $cfg_force_tor = false;

	public function setForceTor($val) {
		$this->cfg_force_tor = $val;
	}

	public function getForceTor() {
		return $this->cfg_force_tor;
	}

	# --- Configuration: useragent

	private $cfg_useragent = 'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:40.0) Gecko/20100101 Firefox/40.1';

	public function setUserAgent($val) {
		$this->cfg_useragent = $val;
	}

	public function getUserAgent() {
		return $this->cfg_useragent;
	}

	# --- Downloader functions (requires ViaThinkSoft vtor from http://www.viathinksoft.de/~daniel-marschall/code/bash/ )

	public function cURLGet($url, $params = '', $fail = true) {
		$params .= " --retry 5 ";
		$params .= " --retry-delay 20 ";
		$params .= " -s "; // silent
#		$params .= " -v "; // verbose
		$params .= " -S "; // but shows errors
		$params .= " -L "; // follow location

		$params .= " -m 120 "; // max 120 secs for everything
		if ((strpos($params, ' -A ') === false) && (strpos($params, ' --user-agent ') === false)) {
			$params .= ' -A '.escapeshellarg($this->getUserAgent());
		}

		if ($fail) $params .= ' -f';

		$cmd = "curl $params ".escapeshellarg($url);
		if ($this->getForceTor()) {
			$cmd = escapeshellcmd($this->vtor_cmd)." -- $cmd";
		}
		$cmd .= " 2> /dev/null";

		$start_t = time();
		$out = array();
		exec($cmd, $out, $code);

		$end_t = time();

		if (($fail) && ($code != 0)) {
			// Dirty workaround for a TOR problem with SSL connections
			// TODO: achtung. endlosschleifen-gefahr
			if ($code == 35) {
				sleep(20);
				return $this->cURLGet($url, $params, $fail);
			}

			# Just for us
#			print_r($out);
#			die('Fetch failure: code '.$code.' after '.($end_t-$start_t)." secs\n");

			$this->lastError = 'Fetch failure: code '.$code.' after '.($end_t-$start_t)." secs";
			return false;
		}

		$out = implode("\n", $out);

		$this->lastError = '';
		return $out;
	}

	protected $lastError = '';

	public function getLastError() {
		# TODO: lieber &$out params bei jeder funktion machen
		return $this->lastError;
	}

	public function cURLDownload($url, $target, $add_params='', $ignore_https_errors=false) {
		if (strpos($add_params, ' -A ') === false) $add_params .= ' -A '.escapeshellarg($this->getUserAgent());

		if ($ignore_https_errors) {
			// Required for some YouTube Videos. Otherwise Code 51 (CN mismatch)
			$add_params .= " -k "; // allow all HTTPS connections, also untrusted ones
		}

		$add_params .= " -L -f -# -o ".escapeshellarg($target).' 2>&1';

		$cmd = 'curl '.escapeshellarg($url).' '.$add_params;

		if ($this->getForceTor()) {
			$cmd = escapeshellcmd($this->vtor_cmd)." -- $cmd";
		}

		$out = array();
		exec($cmd, $out, $code);

		if ($code != 0) {
			// Dirty workaround for a TOR problem with SSL connections
			// TODO: achtung. endlosschleifen-gefahr
			// TODO: diese technik bei fehler 35 auslagern in vdown/vtor?
			if ($code == 35) {
				sleep(20);
				return $this->cURLDownload($url, $target, $add_params, $ignore_https_errors);
			}

			# Just for us
#			print_r($out);
#			die('Fetch failure: code '.$code."\n");

			$this->lastError = 'Fetch failure: code '.$code;
			return false;
		}

		if ($code == 0) {
			$this->lastError = '';
			return true;
		} else {
			$this->lastError = "CMD = $cmd\n"."EXIT CODE = $code\n"."OUTPUT = ".implode("\n", $out);
			return false;
		}
	}

}

function curl_simpe_get($url) {
	$dwn = new VTSDownloader();
	return $dwn->cURLGet($url);
}
